import random
import math
random.seed("basic03/autocompletion")

class TestGen:
    def __init__(self):
        self.test_number = 0

    def generate_samples(self):
        self.print_test(['abacaba', 'ababb', 'abcabac', 'babacb'], 'abacb', 6)
        self.print_test(['exactword'], 'exactword', 4)

    def generate_min_tests(self):
        self.print_test(['a'], 'b', 1000)
        self.print_test(['z'], 'b', 1000)
        self.print_test(['b'], 'b', 1000)
        self.print_test(['z'], 'b', 1)
        self.print_test(['b'], 'b', 1)
        self.print_test(['z'], 'b', 2)
        self.print_test(['b'], 'b', 2)
        self.print_test(['z'], 'b', 3)
        self.print_test(['b'], 'b', 3)

    def generate_hand_tests(self):
        self.print_test(['exactword'], 'exactword', 11)
        self.print_test(['exactword'], 'exactword', 1000)
        self.print_test(['hexactword' * 10], 'hexactword' * 10, 1000)
        self.print_test(['aba', 'bab'], 'abb', 1)
        self.print_test(['aba', 'bab'], 'abb', 2)
        self.print_test(['aba', 'bab'], 'abb', 3)
        self.print_test(['aba', 'bab'], 'abb', 4)

    def random_word(self, length):
        s = ''
        for i in range(length):
            s += chr(ord('a') + random.randint(0, 25))
        return s

    def generate_random_test(self, maxn, maxlen, maxk):
        maxsum = 1000
        s = self.random_word(maxlen)
        words = []
        summary = 0
        for i in range(maxn):
            l = random.randint(0, len(s) - 1)
            r = random.randint(l, min(len(s) - 1, l + min(10, maxlen))) if random.randint(1, 3) != 1 else random.randint(l, min(len(s) - 1, l + min(maxlen, len(s) // 3)))
            word = s[l:r + 1]
            addition = random.randint(0, min(10, maxlen - len(word)))
            for j in range(addition):
                word += chr(random.randint(0, 25) + ord('a'))
            if summary == maxsum:
                break
            if summary + len(word) > maxsum:
                word = word[:maxsum - summary]
            summary += len(word)
            words.append(word)
        k = random.randint(1, maxk)
        self.print_test(words, s, k)

    def generate_max_tests(self, maxn, maxlen, maxs, maxk):
        maxsum = 1000
        basic = 'a' * ((maxsum // 26) - 1)
        words = []
        for i in range(26):
            words.append(basic + str(chr(ord('a') + i)))
        s = 'a' * (maxs - 1) + chr(random.randint(0, 25) + ord('a'))
        self.print_test(words, s, maxk)
        words = []
        for i in range(44): #sqrt(2 * maxsum)
            words.append('a' * (i + 1))
        random.shuffle(words)
        self.print_test(words, 'a', maxk)
        self.print_test(words, 'a' * maxs, maxk)

    def print_test(self, dictionary, s, k):
        self.test_number += 1
        print('Generating test #%d' % self.test_number)

        test_name = "{0:0=2d}".format(self.test_number)
        test_file = open(test_name, 'w')
        print(len(dictionary), file=test_file)
        print('\n'.join(map(str, dictionary)), file=test_file)
        print(s, file=test_file)
        print(k, file=test_file)
        test_file.close()

    def generate_all_tests(self):
        self.generate_samples()
        self.generate_min_tests()
        self.generate_hand_tests()
        
        ns = [2, 3, 5, 10, 50, 100]
        lens = [3, 5, 10, 100]
        maxk = [10, 100, 1000]
        for n in ns:
            for maxlen in lens:
                for k in maxk:
                    self.generate_random_test(n, maxlen, k)
        self.generate_max_tests(100, 100, 100, 1000)

import time
start = time.time()
writer = TestGen()
writer.generate_all_tests()
finish = time.time()
print('All tests were generated. Elapsed time: ', finish - start)
