import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.StringTokenizer;

/**
 * @author Egor Zhdan
 */
public class lightnings_ez {

    BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
    StringTokenizer tokenizer = new StringTokenizer("");

    String nextToken() {
        while (!tokenizer.hasMoreTokens()) {
            try {
                tokenizer = new StringTokenizer(reader.readLine());
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
        }
        return tokenizer.nextToken();
    }

    int nextInt() {
        return Integer.parseInt(nextToken());
    }

    class Request {
        int x1, y1, x2, y2;

        Request(int x1, int y1, int x2, int y2) {
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }
    }

    class DSU {
        int[] parent;
        int[] rank;
        int[] size;
        int cnt;

        DSU(int count, int cnt) {
            this.cnt = cnt;
            parent = new int[count];
            rank = new int[count];
            size = new int[count];
            for (int i = 0; i < parent.length; i++) {
                parent[i] = i;
                size[i] = 1;
            }
        }

        int getRoot(int v) {
            if (parent[v] == v) return v;
            return parent[v] = getRoot(parent[v]);
        }

        void merge(int i, int j) {
            i = getRoot(i);
            j = getRoot(j);
            if (i == j) return;

            if (rank[i] < rank[j]) {
                int tmp = i;
                i = j;
                j = tmp;
            }

            parent[j] = i;
            if (rank[i] == rank[j]) rank[i]++;

            size[i] += size[j];

            if (d[ij(i)[0]][ij(i)[1]] == 0 && d[ij(j)[0]][ij(j)[1]] == 0)
                cnt--;
        }
    }

    final int[] dx = {1, 0, -1, 0};
    final int[] dy = {0, -1, 0, 1};

    int n, m;
    DSU dsu;
    int[][] d;

    int[] ij(int index) {
        return new int[]{index / m, index % m};
    }

    int index(int i, int j) {
        return i * m + j;
    }

    int count() {
        int c = 0;
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (d[i][j] != 0) {
                    c++;
                }
            }
        }
        return dsu.cnt - c;
    }

    void maybeMerge(int i, int j) {
        for (int move = 0; move < 4; move++) {
            int ni = i + dx[move];
            int nj = j + dy[move];
            if (0 <= ni && ni < n && 0 <= nj && nj < m && d[i][j] == 0 && d[ni][nj] == 0) {
                dsu.merge(index(i, j), index(ni, nj));
            }
        }
    }

    void run() {
        n = nextInt();
        m = nextInt();
        int q = nextInt();
        d = new int[n][m];
        Request[] rs = new Request[q];
        for (int i = 0; i < q; i++) {
            int x1 = nextInt() - 1;
            int y1 = nextInt() - 1;
            int x2 = nextInt() - 1;
            int y2 = nextInt() - 1;
            for (int x = x1; x <= x2; x++) {
                d[x][y1]++;
                d[x][y2]++;
            }
            for (int y = y1; y <= y2; y++) {
                d[x1][y]++;
                d[x2][y]++;
            }
            rs[i] = new Request(x1, y1, x2, y2);
        }

        dsu = new DSU(n * m, n * m);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                maybeMerge(i, j);
            }
        }

        int[] ans = new int[q];
        for (int i = rs.length - 1; i >= 0; i--) {
            ans[i] = count();

            int x1 = rs[i].x1;
            int y1 = rs[i].y1;
            int x2 = rs[i].x2;
            int y2 = rs[i].y2;

            for (int x = x1; x <= x2; x++) {
                d[x][y1]--;
                d[x][y2]--;
            }
            for (int y = y1; y <= y2; y++) {
                d[x1][y]--;
                d[x2][y]--;
            }

            for (int x = x1; x <= x2; x++) {
                maybeMerge(x, y1);
                maybeMerge(x, y2);
            }
            for (int y = y1; y <= y2; y++) {
                maybeMerge(x1, y);
                maybeMerge(x2, y);
            }
        }

        StringBuilder sb = new StringBuilder();
        for (int it : ans) sb.append(it).append('\n');
        System.out.println(sb);
    }

    public static void main(String[] args) {
        new lightnings_ez().run();
    }

}

/*

6 6 3
1 1 5 4
3 2 6 5
2 6 4 6

 */